@@ -9,10 +9,10 @@ ResolvingParser that additionally resolves any $ref references.
 """
 
 __author__ = 'Jens Finkhaeuser'
-__copyright__ = 'Copyright (c) 2016-2018 Jens Finkhaeuser'
+__copyright__ = 'Copyright (c) 2016-2020 Jens Finkhaeuser'
 __license__ = 'MIT +no-false-attribs'
 __all__ = ('util', 'mixins', 'cli', 'convert')
-__version__ = '0.17.0'
+__version__ = '0.18.0'
 
 
 # Define our own error class
@@ -221,8 +221,8 @@ class BaseParser(mixins.YAMLMixin, mixins.JSONMixin, object):
 
       try:
         validate_v3_spec(self.specification)
-      except TypeError as type_ex:  # pragma: nocover
-        raise_from(ValidationError, type_ex)
+      except TypeError as type_ex:
+        raise_from(ValidationError, type_ex, self._strict_warning())
       except JSEValidationError as v3_ex:
         raise_from(ValidationError, v3_ex)
       except RefResolutionError as ref_ex:
@@ -234,12 +234,20 @@ class BaseParser(mixins.YAMLMixin, mixins.JSONMixin, object):
       try:
         validate_v2_spec(self.specification)
       except TypeError as type_ex:  # pragma: nocover
-        raise_from(ValidationError, type_ex)
+        raise_from(ValidationError, type_ex, self._strict_warning())
       except JSEValidationError as v2_ex:
         raise_from(ValidationError, v2_ex)
       except RefResolutionError as ref_ex:
         raise_from(ValidationError, ref_ex)
 
+  def _strict_warning(self):
+    """Return a warning if strict mode is off."""
+    if self.options.get('strict', True):
+      return ('Strict mode enabled (the default), so this could be due to an '
+          'integer key, such as an HTTP status code.')
+    return ('Strict mode disabled. Prance cannot help you narrow this further '
+        'down, sorry.')
+
 
 class ResolvingParser(BaseParser):
   """The ResolvingParser extends BaseParser with resolving references."""
@@ -251,7 +259,12 @@ class ResolvingParser(BaseParser):
     Resolves JSON pointers/references (i.e. '$ref' keys) before validating the
     specs. The implication is that self.specfication is fully resolved, and
     does not contain any references.
+
+    Additional parameters, see :py::class:`util.RefResolver`.
     """
+    # Create a reference cache
+    self.__reference_cache = {}
+
     BaseParser.__init__(
         self,
         url = url,
@@ -267,8 +280,16 @@ class ResolvingParser(BaseParser):
     # http://swagger.io/specification/#referenceObject
     # We therefore use our own resolver first, and validate later.
     from .util.resolver import RefResolver
-    encoding = self.options.get('encoding', None)
-    resolver = RefResolver(self.specification, self.url, encoding = encoding)
+    forward_arg_names = ('encoding', 'recursion_limit',
+            'recursion_limit_handler', 'resolve_types')
+    forward_args = {
+      k: v for (k, v) in self.options.items() if k in forward_arg_names
+    }
+    resolver = RefResolver(
+        self.specification, self.url,
+        reference_cache = self.__reference_cache,
+        **forward_args
+      )
     resolver.resolve_references()
     self.specification = resolver.specs
 
